\name{dtweedie}
\alias{dtweedie}
\alias{dtweedie.series}
\alias{dtweedie.inversion}
\alias{ptweedie}
\alias{ptweedie.series}
\alias{ptweedie.inversion}
\alias{qtweedie}
\alias{rtweedie}
\title{Tweedie Distributions}
\description{Density, distribution function, quantile function and random
     generation for the Tweedie distributions}
\usage{
dtweedie(y, power, mu, phi)
dtweedie.series(y, power, mu, phi)
dtweedie.inversion(y, power, mu, phi, exact=FALSE, rotate=TRUE)
ptweedie(q, power, mu, phi)
ptweedie.series(q, power, mu, phi)
qtweedie(p, power, mu, phi)
rtweedie(n, power, mu, phi)
}
\arguments{
\item{y, q}{vector of quantiles}
\item{p}{vector of probabilities}
\item{n}{the number of observations}
\item{power}{the value of \eqn{p}{power} such that the variance is 
\eqn{\mbox{var}[Y]=\phi\mu^p}{var(Y) = phi * mu^power}}
\item{mu}{the mean}
\item{phi}{the dispersion}
\item{exact}{logical flag; 
if \code{TRUE},
the exact zero \eqn{W}{W}-algorithm of Sidi (1982) is used.
If \code{FALSE} (the default),
the (approximate zero) modified \eqn{W}{W}-algorithm of Sidi (1988) is used.
The modified algorithm requires less computation but is often less accurate;
the exact zero \eqn{W}{W}-algorithm can be slower but generally improves accuracy.
}
\item{rotate}{logical flag;
if \code{TRUE} (the default),
the algorithm rotates before inverting the solution, 
which has the effect of increasing the relative accuracy.}
}
\value{
density (\code{dtweedie}),
probability (\code{ptweedie}),
quantile (\code{qtweedie})
or random sample (\code{rtweedie})
for the given Tweedie distribution with parameters 
\code{mu}, 
\code{phi}
and 
\code{power}.
}
\details{
    The Tweedie family of distributions belong to the class
    of exponential dispersion models (\acronym{EDM}s),
    famous for their role in generalized linear models.
    The Tweedie distributions are the \acronym{EDM}s with a variance of the form
    \eqn{\mbox{var}[Y]=\phi\mu^p}{var(Y) = phi * mu^power}
    where \eqn{p}{power} is greater than or equal to one, or less than or equal to zero.
    \bold{This function only evaluates for \eqn{p}{power}
    greater than or equal to one.}
    Special cases include the 
    normal (\eqn{p=0}{power=0}),
    Poisson (\eqn{p=1}{power=1} with \eqn{\phi=1}{phi=1}),
    gamma (\eqn{p=2}{power=2})
    and
    inverse Gaussian (\eqn{p=3}{power=3})
    distributions.
    For other values of  \code{power},
    the distributions are still defined but cannot be written in closed form,
    and hence evaluation is very difficult.

    When \eqn{1<p<2}{1 < power < 2},
    the distribution are continuous for \eqn{Y}{Y} greater than zero,
    with a positive mass at \eqn{Y=0}{Y=0}.
    For \eqn{p>2}{power > 2}, 
    the distributions are continuous for \eqn{Y}{Y} greater than zero.

    This function evaluates the density or cumulative probability 
    using one of two methods, 
    depending on the combination of parameters. 
    One method is the evaluation of an infinite series.
    The second interpolates some stored values computed from a 
    Fourier inversion technique. 
    
    The function \code{dtweedie.inversion}
    evaluates the density using a Fourier series technique; 
    \code{ptweedie.inversion} does likewise for the cumulative 
    probabilities. 
    The actual code is contained in an external FORTRAN program. 
    Different code is used for \eqn{p>2}{power > 2}
    and for \eqn{1<p<2}{1 < power < 2}. 
    
    The function \code{dtweedie.series} evaluates the density
    using a series expansion; 
    a different series expansion
    is used for \eqn{p>2}{power > 2} and for \eqn{1<p<2}{1 < power < 2}.
    The function \code{ptweedie.series} does likewise for the 
    cumulative probabilities but only for \eqn{1<p<2}{1 < power < 2}. 
    
    The function \code{dtweedie} uses a two-dimensional interpolation procedure to 
    compute the density for some parts of the parameter space from 
    previously computed values found from the series or the 
    inversion. For other parts of the parameter space, 
    the series solution is found. 
    
    \code{ptweedie} returns either the computed series 
    solution or inversion solution.
}
\author{Peter Dunn (\email{dunn@usq.edu.au})}
\references{
Dunn, Peter K and Smyth, Gordon K (To appear).
Series evaluation of Tweedie exponential dispersion model densities
\emph{Statistics and Computing}.

Dunn, Peter K and Smyth, Gordon K (2001).
Tweedie family densities: methods of evaluation.
\emph{Proceedings of the 16th International Workshop on Statistical Modelling},
Odense, Denmark, 2--6 July

Jorgensen, B. (1987).
Exponential dispersion models.
\emph{Journal of the Royal Statistical Society}, B,
\bold{49}, 127--162.

Jorgensen, B. (1997).
\emph{Theory of Dispersion Models}.
Chapman and Hall, London.

Sidi, Avram (1982).
The numerical evaluation of very oscillatory infinite integrals by
extrapolation.
\emph{Mathematics of Computation}
\bold{38}(158), 517--529.

Sidi, Avram (1988).  
A user-friendly extrapolation method for
oscillatory infinite integrals.
\emph{Mathematics of Computation}
\bold{51}(183), 249--266.

Tweedie, M. C. K. (1984).
An index which distinguishes between some important exponential families.
\emph{Statistics: Applications and New Directions.
Proceedings of the Indian Statistical Institute Golden Jubilee International Conference}
(Eds. J. K. Ghosh and J. Roy), pp. 579-604. Calcutta: Indian Statistical Institute.
}
\seealso{\code{\link{dtweedie.saddle}} }

\examples{
### Plot a Tweedie density
power <- 2.5
mu <- 1 
phi <- 1 
y <- seq(0, 10, length=100) 
fy <- dtweedie( y=y, power=power, mu=mu, phi=phi) 
plot(y, fy, type="l")
# Compare to the saddlepoint density
f.saddle <- dtweedie.saddle( y=y, power=power, mu=mu, phi=phi) 
lines( y, f.saddle, col=2 )

### A histogram of Tweedie random numbers
hist( rtweedie( 1000, power=1.2, mu=1, phi=1) )

### An example of the multimodal feature of the Tweedie
### family with power near 1 (from the first reference
### listed above).
y <- seq(0.001,2,len=1000)
mu <- 1
phi <- 0.1
p <- 1.02
f1 <- dtweedie(y,mu=mu,phi=phi,power=p)
plot(y, f1, type="l", xlab="y", ylab="Density")
p <- 1.05
f2<- dtweedie(y,mu=mu,phi=phi,power=p)
lines(y,f2, col=2)
} 
\keyword{models}


\eof
\name{dtweedie.saddle}
\alias{dtweedie.saddle}
\title{Tweedie Distributions (saddlepoint approximation)}
\description{Saddlepoint density for the Tweedie distributions}
\usage{dtweedie.saddle(y, power, mu, phi, eps=1/6)}
\arguments{
\item{y}{the vector of responses}
\item{power}{the value of \eqn{p}{power} such that the variance is \eqn{\mbox{var}[Y]=\phi\mu^p}{var(Y) = phi * mu^power}}
\item{mu}{the mean}
\item{phi}{the dispersion}
\item{eps}{the offset in computing the variance function.
The default is \code{eps=1/6}
(as suggested by Nelder and Pregibon, 1987).}
}
\value{saddlepoint (approximate) density
for the given Tweedie distribution with parameters 
\code{mu},
\code{phi}
and
\code{power}.
}
\details{
    The Tweedie family of distributions belong to the class
    of exponential dispersion models (\acronym{EDM}s),
    famous for their role in generalized linear models.
    The Tweedie distributions are the \acronym{EDM}s with a variance of the form
    \eqn{\mbox{var}[Y]=\phi\mu^p}{var(Y) = phi * mu^power}
    where \eqn{p}{power} is greater than or equal to one, or less than or equal to zero.
    \bold{This function only evaluates for \eqn{p}{power}
    greater than or equal to one.}
    Special cases include the 
    normal (\eqn{p=0}{power=0}),
    Poisson (\eqn{p=1}{power=1} with \eqn{\phi=1}{phi=1}),
    gamma (\eqn{p=2}{power=2})
    and
    inverse Gaussian (\eqn{p=3}{power=3})
    distributions.
    For other values of  \code{power},
    the distributions are still defined but cannot be written in closed form,
    and hence evaluation is very difficult.
    
    When \eqn{1<p<2}{1 < power < 2},
    the distribution are continuous for \eqn{Y}{Y} greater than zero,
    with a positive mass at \eqn{Y=0}{Y=0}.
    For \eqn{p>2}{power > 2}, 
    the distributions are continuous for \eqn{Y}{Y} greater than zero.

    This function approximates the density using the
    saddlepoint approximation defined by Nelder and Pregibon (1987).
}
\author{Peter Dunn (\email{dunn@usq.edu.au})}
\references{
Daniels, H. E. (1954).
Saddlepoint approximations in statistics.
\emph{Annals of Mathematical Statistics},
\bold{25}(4), 631--650.

Daniels, H. E. (1980).
Exact saddlepoint approximations.
\emph{Biometrika},
\bold{67}, 59--63.

Dunn, Peter K and Smyth, Gordon K (2001).
Tweedie family densities: methods of evaluation.
\emph{Proceedings of the 16th International Workshop on Statistical Modelling},
Odense, Denmark, 2--6 July

Jorgensen, B. (1987).
Exponential dispersion models.
\emph{Journal of the Royal Statistical Society}, B,
\bold{49}, 127-162.

Jorgensen, B. (1997).
\emph{Theory of Dispersion Models},
Chapman and Hall, London.

Nelder, J. A. and Pregibon, D. (1987).
An extended quasi-likelihood function.
\emph{Biometrika},
\bold{74}(2), 221--232.

Tweedie, M. C. K. (1984). 
An index which distinguishes between some important exponential families.
\emph{Statistics: Applications and New Directions.
Proceedings of the Indian Statistical Institute Golden Jubilee International Conference}
(Eds. J. K. Ghosh and J. Roy), pp. 579-604. Calcutta: Indian Statistical Institute.
}
\seealso{\code{\link{dtweedie}} }

\examples{
p <- 2.5
mu <- 1
phi <- 1
y <- seq(0, 10, length=100)
fy <- dtweedie( y=y, power=p, mu=mu, phi=phi)
plot(y, fy, type="l")
# Compare to the saddlepoint density
f.saddle <- dtweedie.saddle( y=y, power=p, mu=mu, phi=phi)
lines( y, f.saddle, col=2 )
}
\keyword{models}

\eof
\name{dtweedie-internal}
\alias{dtweedie.dldphi}
\alias{dtweedie.dldphi.saddle}
\alias{dtweedie.dlogfdphi}
\alias{dtweedie.logl}
\alias{dtweedie.logl.saddle}
\alias{dtweedie.logv.bigp}
\alias{dtweedie.logw.smallp}
\alias{dtweedie.interp}
\alias{dtweedie.jw.smallp}
\alias{dtweedie.kv.bigp}
\alias{dtweedie.series.bigp}
\alias{dtweedie.series.smallp}
\alias{tweedie.dev}
\alias{stored.grids}

\title{Tweedie internal function}

\description{Internal tweedie functions.}

\usage{
dtweedie.dldphi( phi, mu, power, y)
dtweedie.dldphi.saddle( y, mu, phi, power)
dtweedie.dlogfdphi(y, mu, phi, power)
dtweedie.logl(phi, y, mu, power)
dtweedie.logl.saddle( phi, power, y, mu, eps=0)
dtweedie.logv.bigp( y, phi, power)
dtweedie.logw.smallp(y, phi, power)
dtweedie.interp(grid, nx, np, xix.lo, xix.hi,p.lo, p.hi, power, xix)
dtweedie.jw.smallp(y, phi, power )
dtweedie.kv.bigp(y, phi, power)
dtweedie.series.bigp(power, y, mu, phi)
dtweedie.series.smallp(power, y, mu, phi)
tweedie.dev(y, mu, power)
stored.grids(power)
}

\arguments{
\item{y}{the vector of responses}
\item{power}{the value of \eqn{p}{power} such that the variance is
\eqn{\mbox{var}[Y]=\phi\mu^p}{var(Y) = phi * mu^power}}
\item{mu}{the mean}
\item{phi}{the dispersion}
\item{grid}{the interpolation grid necessary for the given value of \eqn{p}{power}}
\item{nx}{the number of interpolation points in the \eqn{\xi}{xi} dimension}
\item{np}{the number of interpolation points in the \eqn{p}{power} dimension}
\item{xix.lo}{the lower value of the transformed \eqn{\xi}{xi} value used in the interpolation grid.
(Note that the value of \eqn{\xi}{xi} is from \eqn{0} to \eqn{\infty}{infty},
and is transformed such that it is on the range \eqn{0} to \eqn{1}.)}
\item{xix.hi}{the higher value of the transformed \eqn{\xi}{xi} value used in the interpolation grid.}
\item{p.lo}{the lower value of \eqn{p} value used in the interpolation grid.}
\item{p.hi}{the higher value of \eqn{p} value used in the interpolation grid.}
\item{xix}{the value of the transformed \eqn{\xi}{xi} at which a value is sought.}
\item{eps}{the offset in computing the variance function in the saddlepoint approximation.
The default is \code{eps=1/6}
(as suggested by Nelder and Pregibon, 1987).}
}
\details{
These are not to be called by the user.
}

\author{Peter Dunn (\email{dunn@usq.edu.au})}
\references{
Nelder, J. A. and Pregibon, D. (1987).
An extended quasi-likelihood function
\emph{Biometrika},
\bold{74}(2), 221--232.
}
\keyword{models}

\eof
\name{tweedie.profile}
\alias{tweedie.profile}
\title{Tweedie Distributions: mle estimation of p}
\description{Maximum likelihood estimation of the Tweedie index parameter \eqn{p}{power}.}
\usage{tweedie.profile(formula, p.vec, smooth=FALSE, do.plot=FALSE, do.ci=smooth,
eps=1/6, do.points=do.plot, method="series", conf.level=0.95, 
phi.method=ifelse(method == "saddlepoint", "saddlepoint", "mle"))}
\arguments{
\item{formula}{a formula expression as for other regression models and generalized linear models, 
of the form \code{response ~ predictors}. 
For details, 
see the documentation for \code{\link{lm}}, 
\code{\link{glm}} and \code{\link{formula}}}
\item{p.vec}{a vector of \code{p} values for consideration.
The values must all be larger than one
(if the response variable has exact zeros,
the values must all be between one and two).
See the DETAILS section below for further details.}
\item{smooth}{logical flag.
If \code{TRUE},
a spline is fitted to the data to smooth the profile likelihood plot.
If \code{FALSE} (the default),
no smoothing is used 
(and the function is quicker).
\bold{Note} that \code{p.vec} must contain \emph{at least five points}
for smoothing to be allowed.}
\item{do.plot}{logical flag.
If \code{TRUE},
a plot of the profile likelihood is produce.
If \code{FALSE} (the default),
no plot is produced.}
\item{do.ci}{logical flag.
If \code{TRUE},
the nominal 100*\code{conf.level}
is computed.
If \code{FALSE},
the confidence interval is not computed.
By default,
\code{do.ci} is the same value as \code{smooth},
since a confidence interval will only be accurate if
smoothing has been performed.
Indeed,
if \code{smooth=FALSE},
confidence intervals are never computed and
\code{do.ci} is forced to \code{FALSE} if it is given as \code{TRUE}.}
\item{eps}{the offset in computing the variance function.
The default is \code{eps=1/6}
(as suggested by Nelder and Pregibon, 1987).
Note \code{eps} is ignored unless the 
\code{method="saddlepoint"}
as it makes no sense.}
\item{do.points}{plot the points on the plot where the
(log-) likelihood is computed for the given values of \code{p};
defaults to the same value as \code{do.plot}}
\item{method}{the method for computing the (log-) likelihood.
One of
\code{"series"} (the default),
\code{"inversion"},
\code{"interpolation"}
or
\code{"saddlepoint"}.
If there are any troubles using this function,
often a change of method will fix the problem.
Note that \code{method="saddlepoint"}
is only an approximate method for computing the (log-) likelihood.
Using \code{method="interpolation"}
may produce a jump in the profile likelihood as it changes computational regimes.}
\item{conf.level}{the confidence level for the computation of the nominal
confidence interval.
The default is \code{conf.level=0.95}.}
\item{phi.method}{the method for estimating \code{phi},
one of
\code{"saddlepoint"}
or
\code{"mle"}.
A maximum likelihood estimate is used unless
\code{method="saddlepoint"},
when the saddlepoint approximation method is used.
Note that using 
\code{phi.method="saddlepoint"}
is equivalent to using the mean deviance estimator of \code{phi}.
}
}
\value{
A list containing the components:
\code{y} and \code{x}
(such that \code{plot(x,y)} (partially)
recreates the profile likelihood plot);
\code{ht} (the height of the nominal confidence interval);
\code{L} (the estimate of the (log-) likelihood at each given value of \code{p});
\code{p} (the \code{p}-values used);
\code{p.max} (the estimate of the mle of \code{p});
\code{L.max} (the estimate of the (log-) likelihood at \code{p.max});
\code{phi} (the estimate of \code{phi} at \code{p.max});
\code{ci} (the lower and upper limits of the confidence interval for \code{p});
\code{method} (the method used).
}
\note{The estimates of \code{p}
and \code{phi} are printed.
The result is printed invisibly.

If the response variable has any exact zeros,
the values in \code{p.vec}
must all be between one and two.

The function is sometimes unstable and may fail.
It may also be very slow.
One solution is to change the method.
The default is \code{method="series"} (the default);
then try \code{method="inversion"},
\code{method="interpolation"}
and
\code{method="saddlepoint"}
in that order.
Note that 
\code{method="saddlepoint"}
is an approximate method only.
Also make sure the values in \code{p.vec}
are suitable for the data  
(see above paragraph).

It is recommended that for the first use with a data set,
use \code{p.vec} with only a small number of values
and set
\code{smooth=FALSE},
\code{do.ci=FALSE}.
If this is successful,
a larger vector \code{p.vec}
and smoothing can be used.}
\details{
For each value in \code{p.vec},
the function computes an estimate of \code{phi}
and then computes the value of the log-likelihood for these parameters.
The plot of the log-likelihood against \code{p.vec} 
allows the maximum likelihood value of \code{p}
to be found.
Once the value of \code{p} is found,
the distribution within the class of Tweedie distribution is identified.
}
\author{Peter Dunn (\email{dunn@usq.edu.au})}
\references{
Dunn, Peter K and Smyth, Gordon K (2001).
Tweedie family densities: methods of evaluation.
\emph{Proceedings of the 16th International Workshop on Statistical Modelling},
Odense, Denmark, 2--6 July

Jorgensen, B. (1987).
Exponential dispersion models.
\emph{Journal of the Royal Statistical Society}, B,
\bold{49}, 127--162.

Jorgensen, B. (1997).
\emph{Theory of Dispersion Models}.
Chapman and Hall, London.

Nelder, J. A. and Pregibon, D. (1987).
An extended quasi-likelihood function.
\emph{Biometrika}
\bold{74}(2),
221--232.

Tweedie, M. C. K. (1984).
An index which distinguishes between some important exponential families.
\emph{Statistics: Applications and New Directions.
Proceedings of the Indian Statistical Institute Golden Jubilee International Conference}
(Eds. J. K. Ghosh and J. Roy), pp. 579-604. Calcutta: Indian Statistical Institute.
}
\seealso{
\code{\link{dtweedie}}
\code{\link{dtweedie.saddle}}
}
\examples{
library(statmod) # Needed to use  tweedie.profile
# Generate some fictitious data
test.data <- rgamma(n=200, scale=1, shape=1)
# The gamma is a Tweedie distribution with power=2;
# let's see if the profile plot shows this
out <- tweedie.profile( test.data ~1, p.vec=seq(1.7, 2.3, length=10),
       do.plot=TRUE, method="interpolation", smooth=TRUE, do.ci=TRUE)
# And plot the points to see how the smooth went
points( out$p, out$L)
}

\keyword{models}

\eof
